//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "ObjGraf.h"
#include <syncobjs.hpp>
//---------------------------------------------------------------------------
#pragma package(smart_init)

//Seccin crtica

TCriticalSection *SC = new TCriticalSection();

// Evento

TEvent *event = new TEvent (NULL, true, false, "Evento Pelota");

// Mtodos de TObjGraf

TObjGraf::TObjGraf (TPaintBox *_PaintBox, TColor _Color, int _X, int _Y)
{
  PaintBox = _PaintBox;
  Color     = _Color;
  FX        = _X;
  FY        = _Y;
};

TObjGraf::TObjGraf (TObjGraf *ObjGraf)
{
  PaintBox = ObjGraf->PaintBox;
  Color     = ObjGraf->Color;
  FX        = ObjGraf->FX;
  FY        = ObjGraf->FY;
};

void TObjGraf::SetX (int _X) throw (EFueraRango)
{
  if (_X < 0) {
    _X = 0;
    throw EFueraRango();
  } else if (_X > (PaintBox->Width - Ancho)) {
    FX = (PaintBox->Width - Ancho);
    throw EFueraRango();
  } else
    FX = _X;
};

void TObjGraf::SetY (int _Y) throw (EFueraRango)
{
  if (_Y < 0) {
    FY = 0;
    throw EFueraRango();
  } else if (_Y > (PaintBox->Height - Alto)) {
    FY = (PaintBox->Height - Alto);
    throw EFueraRango();
  } else
    FY = _Y;
};


// Mtodos de TCirculo

TCirculo::TCirculo (TPaintBox *_PaintBox, TColor _Color, int _X, int _Y,  int _Radio)
  : TObjGraf (_PaintBox, _Color, _X, _Y)
{
  Radio = _Radio;
};

void TCirculo::Mostrar (void)
{
  PaintBox->Canvas->Pen->Color   = Color;
  PaintBox->Canvas->Brush->Color = Color;
  PaintBox->Canvas->Ellipse (X, Y, X+Radio*2, Y+Radio*2);
};


// Mtodos TPelota

TPelota::TPelota (TPaintBox *_PaintBox, TColor _Color, int _Radio,
  int _X, int _Y, TDireccion _Direccion, int _Velocidad)
  : TCirculo (_PaintBox, _Color, _Radio, _X, _Y)
{
  Direccion = _Direccion;
  Velocidad = _Velocidad;
};

void TPelota::Mostrar(void)
{
  SC->Acquire();

  try {
    PaintBox->Canvas->Pen->Color   = clBlack;
    PaintBox->Canvas->Brush->Color = Color;
    PaintBox->Canvas->Ellipse (X, Y, X+Radio*2, Y+Radio*2);
  } __finally {
    SC->Release();
  }
};

void TPelota::Borrar(void)
{
  SC->Acquire();

  try {
    PaintBox->Canvas->Pen->Color   = PaintBox->Color;
    PaintBox->Canvas->Brush->Color = PaintBox->Color;
    PaintBox->Canvas->Ellipse (X, Y, X+Radio*2, Y+Radio*2);
  } __finally {
    SC->Release();
  }
};

void __fastcall TPelota::Mover(void)
{
  bool wall = false;


  Borrar();

  try {
    X += FDirX * Velocidad;
  } catch (EFueraRango) {
    FDirX = -FDirX;
    wall = true;
  };

  try {
    Y += FDirY * Velocidad;
  } catch (EFueraRango) {
    FDirY = -FDirY;
    wall = true;
  };


  Mostrar();

  if (wall) { // Evento
     event->SetEvent();
     Sleep(2000);
     event->WaitFor(1000);
     event->ResetEvent();
  }
};

void TPelota::SetDireccion (TDireccion _Dir)
{
  FDirY = (_Dir & 1)? +1 :((_Dir & 2)? -1 : 0);
  FDirX = (_Dir & 4)? +1 :((_Dir & 8)? -1 : 0);
};

TDireccion TPelota::GetDireccion (void)
{
  TDireccion _Dir;

  _Dir =(TDireccion)(        (FDirY == +1)? 1 : ((FDirY == -1)? 2 : 0));
  _Dir =(TDireccion)( _Dir + (FDirX == +1)? 4 : ((FDirX == -1)? 8 : 0));

  return _Dir;
};


// Mtodos de TCuadrado

TCuadrado::TCuadrado (TPaintBox *_PaintBox, TColor _Color,
  int _X, int _Y,  int _Lado)
  : TObjGraf (_PaintBox, _Color, _X, _Y)
{
  Lado = _Lado;
};

void TCuadrado::Mostrar (void)
{
  PaintBox->Canvas->Pen->Color   = Color;
  PaintBox->Canvas->Brush->Color = Color;
  PaintBox->Canvas->Rectangle (X, Y, X+Lado, Y+Lado);
};
